package cn.soul2.jyjc.admin.controller;

import cn.soul2.jyjc.admin.annotation.SkinLogin;
import cn.soul2.jyjc.admin.dto.QrDTO;
import cn.soul2.jyjc.admin.entity.QrDO;
import cn.soul2.jyjc.admin.entity.QuestionnaireDO;
import cn.soul2.jyjc.admin.repository.IQrRepository;
import cn.soul2.jyjc.admin.repository.IQuestionnaireRepository;
import cn.soul2.jyjc.admin.utils.base.BackUtils;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.QrVO;
import cn.soul2.jyjc.admin.vo.base.Back;
import cn.soul2.jyjc.admin.vo.base.VPage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author Soul2
 * @date 2024-03-11 15:16
 */

@Slf4j
@RestController
@RequestMapping("/qr")
public class QrController {

    @Autowired
    IQrRepository qrRepository;

    @Autowired
    private IQuestionnaireRepository questionnaireRepository;

    @PostMapping("/page")
    Back<VPage<QrVO>> page(@RequestBody QrDTO dto) {
        VPage<QrDO> page = qrRepository.page(dto);
        Map<String, QuestionnaireDO> map = questionnaireRepository.mapByIds(page.getRows().stream().map(QrDO::getQnId).collect(Collectors.toSet()));
        VPage<QrVO> toPage = PageUtils.to(page, page.getRows().stream().map(e -> {
            QrVO vo = new QrVO();
            BeanUtils.copyProperties(e, vo);
            if (StringUtils.isNotBlank(vo.getQnId())) {
                vo.setQnName(map.getOrDefault(vo.getQnId(), null).getName());
            }
            return vo;
        }).collect(Collectors.toList()));
        return BackUtils.success(toPage);
    }

    @PostMapping("/saveOrUpdate")
    Back<Boolean> saveOrUpdate(@RequestBody List<QrVO> dtos) {
        return BackUtils.success(qrRepository.saveOrUpdate(dtos));
    }

    @PostMapping("/remove")
    Back<Boolean> remove(@RequestBody List<String> ids) {
        return BackUtils.success(qrRepository.remove(ids));
    }

    @PostMapping("status")
    public Back<Boolean> status(@RequestBody QrDTO dto) {
        return BackUtils.success(qrRepository.status(dto));
    }

    @SkinLogin
    @PostMapping("to/{query}")
    public Back<String> qrQuery(@PathVariable String query) {
        return BackUtils.success(qrRepository.getTo(query));
    }

}
