package cn.soul2.jyjc.admin.controller;

import cn.soul2.jyjc.admin.annotation.SkinLogin;
import cn.soul2.jyjc.admin.dto.GetSubjectDTO;
import cn.soul2.jyjc.admin.dto.QnSubjectRefItemDTO;
import cn.soul2.jyjc.admin.dto.SubjectDTO;
import cn.soul2.jyjc.admin.dto.base.UpdateStatusDTO;
import cn.soul2.jyjc.admin.entity.QuestionnaireDO;
import cn.soul2.jyjc.admin.entity.SubjectItemsDO;
import cn.soul2.jyjc.admin.entity.sqlresult.SubjectChooseListDO;
import cn.soul2.jyjc.admin.repository.IQuestionnaireRepository;
import cn.soul2.jyjc.admin.repository.ISubjectItemsRepository;
import cn.soul2.jyjc.admin.repository.ISubjectRepository;
import cn.soul2.jyjc.admin.service.ISubjectService;
import cn.soul2.jyjc.admin.utils.base.BackUtils;
import cn.soul2.jyjc.admin.vo.SubjectItemVO;
import cn.soul2.jyjc.admin.vo.SubjectVO;
import cn.soul2.jyjc.admin.vo.base.Back;
import cn.soul2.jyjc.admin.vo.base.VPage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Soul2
 * @date 2024-03-14 17:36
 */

@Slf4j
@RestController
@RequestMapping("/subject")
public class SubjectController {

    @Autowired
    private IQuestionnaireRepository questionnaireRepository;

    @Autowired
    private ISubjectRepository subjectRepository;

    @Autowired
    private ISubjectItemsRepository itemsRepository;

    @Autowired
    private ISubjectService subjectService;

    @PostMapping("saveOrUpdate")
    public Back<Boolean> saveOrUpdate(@RequestBody SubjectDTO dto) {
        return BackUtils.success(subjectRepository.saveOrUpdate(dto));
    }

    @PostMapping("page")
    public Back<VPage<SubjectVO>> page(@RequestBody SubjectDTO dto) {
        return BackUtils.success(subjectRepository.page(dto));
    }

    @PostMapping("remove")
    public Back<Boolean> remove(Collection<String> ids) {
        return BackUtils.success(subjectRepository.removeAndItems(ids));
    }

    @PostMapping("status")
    public Back<Boolean> status(@RequestBody UpdateStatusDTO dto) {
        return BackUtils.success(subjectRepository.status(dto));
    }

    @PostMapping("items")
    public Back<SubjectVO> item(@RequestBody SubjectDTO dto) {
        SubjectVO vo = new SubjectVO();
        BeanUtils.copyProperties(dto, vo);
        List<SubjectItemsDO> items = itemsRepository.listBySubjectId(dto.getId());
        return BackUtils.success(vo.setItems(items.stream().map(o -> {
            SubjectItemVO itemVo = new SubjectItemVO();
            itemVo.setItemId(o.getId());
            BeanUtils.copyProperties(o, itemVo);
            return itemVo;
        }).collect(Collectors.toList())));
    }

    @PostMapping("chooseList")
    public Back<VPage<SubjectChooseListDO>> chooseList(@RequestBody QnSubjectRefItemDTO dto) {
        return BackUtils.success(subjectRepository.chooseList(dto));
    }

    @PostMapping("getSubject")
    @SkinLogin
    public Back<List<SubjectVO>> getSubject(@RequestBody GetSubjectDTO dto) {
        if (StringUtils.isBlank(dto.getQnId())) {
            QuestionnaireDO qn = questionnaireRepository.getTheLatestOne();
            if (qn != null) {
                dto.setQnId(qn.getId());
            }
        }
        return BackUtils.success(subjectService.getSubject(dto));
    }

}
