package cn.soul2.jyjc.admin.filter;

import cn.soul2.jyjc.admin.utils.EncryptUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.http.fileupload.servlet.ServletFileUpload;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * @author shaoduo
 * @description 替换HttpServletRequest
 * @since 1.0
 **/
@Slf4j
public class ReplaceStreamFilter implements Filter {
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        log.info("StreamFilter初始化...");
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        //如果是文件上传则会报错可以判断是否是文件上传不读取流即可
        if (ServletFileUpload.isMultipartContent((HttpServletRequest) request)) {
            chain.doFilter(request, response);
        } else if ("OPTIONS".equalsIgnoreCase(((HttpServletRequest) request).getMethod())) {
            chain.doFilter(request, response);
        } else {
            ServletRequest requestWrapper = new ShaoduoRequestWrapper((HttpServletRequest) request);
//            System.out.printf("ReplaceStreamFilter触发, Method: %s, URI: %s%n", ((HttpServletRequest) request).getMethod(), ((HttpServletRequest) request).getRequestURI());
            HttpServletResponse httpResponse = (HttpServletResponse) response;
            Soul2ResponseWrapper soul2ResponseWrapper = new Soul2ResponseWrapper(httpResponse);
            chain.doFilter(requestWrapper, soul2ResponseWrapper);
            try {
                byte[] data = soul2ResponseWrapper.getResponseData();
                log.debug("原始返回 -> " + new String(data));
                String encryptBody = EncryptUtils.encrypt(new String(data));
                log.debug("加密返回 -> " + encryptBody);
                soul2ResponseWrapper.setHeader("encrypt", "1");
                soul2ResponseWrapper.setHeader("Access-Control-Expose-Headers", "encrypt");
                PrintWriter out = response.getWriter();
                out.print(encryptBody);
                out.flush();
                out.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }

    @Override
    public void destroy() {
        log.info("StreamFilter销毁...");
    }
}
