package cn.soul2.jyjc.admin.interceptor;

import cn.soul2.jyjc.admin.annotation.SkinEncrypt;
import cn.soul2.jyjc.admin.annotation.SkinLogin;
import cn.soul2.jyjc.admin.bean.UserLoginStatusBean;
import cn.soul2.jyjc.admin.filter.ShaoduoRequestWrapper;
import cn.soul2.jyjc.admin.service.IUserService;
import cn.soul2.jyjc.admin.utils.EncryptUtils;
import cn.soul2.jyjc.admin.vo.base.Back;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.lang.reflect.Method;

/**
 * Token拦截器
 *
 * @author Soul2
 * @date 2024-04-02 15:31
 */
@Slf4j
public class FinallyInterceptor implements HandlerInterceptor {

    @Resource
    private UserLoginStatusBean userLoginStatusBean;

    @Autowired
    private IUserService userService;

    private boolean load = true;

    private void loadUserLoginStatus() {
        userLoginStatusBean.loadUserLoginStatus(userService.getLoginedUser());
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (load) {
            loadUserLoginStatus();
            load = false;
        }
        boolean pass = false;
        // 允许OPTIONS请求通过
        if ("OPTIONS".equalsIgnoreCase(request.getMethod())) {
            return true;
        }
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        String httpMethod = request.getMethod();
        SkinEncrypt skinEncrypt = handlerMethod.getMethodAnnotation(SkinEncrypt.class);
        // 检查方法上是否存在SkinLogin注解
        boolean hasSkinLogin = handlerMethod.getMethodAnnotation(SkinLogin.class) != null;
        // 从请求头中获取 token
        String token = request.getHeader("jyjc-Token");
        try {
        /*
            加解密
         */
            // 不拦截get请求
            if ("GET".equals(httpMethod)) {
                pass = true;
                // 如果是post请求且是json
            } else if ("POST".equals(httpMethod)) {
                // 如果类型为空就放行
                if (request.getContentType() == null) {
                    pass = true;
                }
            }
            // 跳过使用 @SkinEncrypt 的情况
            if (skinEncrypt != null) {
                pass = true;
            }

            if (!pass) {
                ShaoduoRequestWrapper shaoduoRequestWrapper;
                try {
                    shaoduoRequestWrapper = (ShaoduoRequestWrapper) request;
                } catch (Exception e) {
                    System.out.printf("request.ClassTypeError: %s%n", request.getClass().getName());
                    return true;
                }
                String sourceParamBody = shaoduoRequestWrapper.getBodyString();
                String afterBody = EncryptUtils.decrypt(sourceParamBody);
                shaoduoRequestWrapper.setBody(afterBody);
                log.debug(String.format("解密: %s -> %s", sourceParamBody, afterBody));

            }

            /* token验证 */
            if (hasSkinLogin) {
                // 跳过使用 @SkinLogin 的情况
                pass = true;
            } else {
                // 验证token
                // 检查 token 是否存在并且有效
                if (token == null) {
                    // 没有Token，拒绝请求
                    response.setStatus(40401);
                    pass = false;
                } else if (userLoginStatusBean != null) {
                    if (!userLoginStatusBean.containsToken(token)) {
                        // Token 无效，拒绝请求
                        response.setContentType("application/json");
                        Back<String> back = new Back<String>().setCode(40401).setMessage("Token invalid!");

                        // 转换为 JSON 字符串
                        ObjectMapper objectMapper = new ObjectMapper();
                        String responseBody = objectMapper.writeValueAsString(back);

                        // 输出错误信息到响应中
                        PrintWriter writer = response.getWriter();
                        writer.print(responseBody);
                        writer.flush();
                        pass = false;
                    } else {
                        // token存在, 通过拦截器
                        pass = true;
                    }
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return pass;
    }
}
