package cn.soul2.jyjc.admin.repository;

import cn.soul2.jyjc.admin.dto.SubjectItemRefDTO;
import cn.soul2.jyjc.admin.entity.RefSubjectItemsDO;
import cn.soul2.jyjc.admin.entity.SubjectItemsDO;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.Collection;
import java.util.List;

/**
 * <p>
 * 题目-选项连接表 服务类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
public interface IRefSubjectItemsRepository extends IService<RefSubjectItemsDO> {

    /**
     * 通过题目id查询选项
     *
     * @param subjectIds 题目id
     * @return {@link List}<{@link SubjectItemsDO}>
     */
    List<SubjectItemsDO> getSubjectItems(Collection<String> subjectIds);

    /**
     * 更新连接
     * 不在新列表中的则删除,无则
     *
     * @param subjectId
     * @param newItemIds
     * @return {@link Integer}
     */
    Integer updateRef(String subjectId, Collection<String> newItemIds);

    /**
     * 连接题目-选项
     *
     * @param dto SubjectId and Subject items Id
     * @return {@link Boolean}
     */
    Boolean connectItems(SubjectItemRefDTO dto);

    /**
     * 解除连接
     *
     * @param itemIds    选项id
     * @param andDestroy 同时删除选项记录
     * @return {@link Boolean}
     */
    Boolean disConnectItems(Collection<String> itemIds, Boolean andDestroy);

    /**
     * 解除连接
     *
     * @param subjectId  题目id
     * @param andDestroy 同时删除选项记录
     * @return {@link Boolean}
     */
    Boolean disConnectItems(String subjectId, Boolean andDestroy);

}
