package cn.soul2.jyjc.admin.repository;

import cn.soul2.jyjc.admin.dto.UserLoginDTO;
import cn.soul2.jyjc.admin.dto.UserLoginOutPageDTO;
import cn.soul2.jyjc.admin.dto.UserLogoutDTO;
import cn.soul2.jyjc.admin.entity.UserDO;
import cn.soul2.jyjc.admin.entity.UserLoginOutDO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 用户登入登出表 服务类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-29 20:46:42
 */
public interface IUserLoginOutRepository extends IService<UserLoginOutDO> {

    /**
     * 分页查询
     *
     * @param dto 查询条件
     * @return {@link IPage}<{@link UserLoginOutDO}>
     */
    IPage<UserLoginOutDO> page(UserLoginOutPageDTO dto);

    /**
     * 用户登录
     *
     * @param dto  用户信息
     * @param user 用户
     * @return {@link String}
     */
    String login(UserLoginDTO dto, UserDO user);

    /**
     * 用户登出
     *
     * @param dto 用户信息
     * @return {@link String}
     */
    String logout(UserLogoutDTO dto);

    /**
     * 用户登出
     *
     * @param userId 用户id
     * @return {@link String}
     */
    Boolean logoutByUserId(String userId);

    /**
     * 删除目标用户的所有登录记录
     *
     * @param userId 用户id
     * @return {@link Boolean}
     */
    Boolean removeByUserId(String userId);

    /**
     * 重建当前登录的用户
     *
     * @return {@link List}<{@link UserLoginOutDO}>
     */
    List<UserLoginOutDO> getLoginedUser();

}
