package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.AnswerPageQueryDTO;
import cn.soul2.jyjc.admin.entity.*;
import cn.soul2.jyjc.admin.mapper.AnswerSheetMapper;
import cn.soul2.jyjc.admin.repository.*;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.AnswerDetailsVO;
import cn.soul2.jyjc.admin.vo.AnswerSheetVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.transaction.Transactional;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 答卷表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
@Service
public class AnswerSheetRepositoryImpl extends ServiceImpl<AnswerSheetMapper, AnswerSheetDO> implements IAnswerSheetRepository {

    @Autowired
    private IQuestionnaireRepository questionnaireRepository;

    @Autowired
    private IQrRepository qrRepository;

    @Autowired
    private IAnswerDetailsRepository detailsRepository;

    @Autowired
    private ISubjectItemsRepository subjectItemsRepository;

    @Autowired
    private ISubjectRepository subjectRepository;

    @Override
    public VPage<AnswerSheetVO> page(AnswerPageQueryDTO dto) {
        LambdaQueryWrapper<AnswerSheetDO> query = Wrappers.lambdaQuery();
        query.eq(StringUtils.isNotBlank(dto.getRespondent()), AnswerSheetDO::getRespondent, dto.getRespondent())
                .eq(dto.getStatus() != null, AnswerSheetDO::getStatus, dto.getStatus())
                .eq(StringUtils.isNotBlank(dto.getQnId()), AnswerSheetDO::getQnId, dto.getQnId())
                .orderByDesc(AnswerSheetDO::getUpdatedTime).orderByDesc(AnswerSheetDO::getCreatedTime);
        VPage<AnswerSheetDO> page = PageUtils.to(super.page(PageUtils.build(dto), query));
        List<AnswerSheetDO> rows = page.getRows();
        Set<String> qnSet = rows.stream().map(AnswerSheetDO::getQnId).collect(Collectors.toSet());
        Set<String> qrSet = rows.stream().map(AnswerSheetDO::getQrId).collect(Collectors.toSet());

        Map<String, QuestionnaireDO> qnMap;
        if (CollectionUtils.isNotEmpty(qnSet)) {
            qnMap = questionnaireRepository.listByIds(qnSet).stream().collect(Collectors.toMap(QuestionnaireDO::getId, o -> o, (o1, o2) -> o1));
        } else {
            qnMap = new HashMap<>();
        }
        Map<String, QrDO> qrMap;
        if (CollectionUtils.isNotEmpty(qrSet)) {
            qrMap = qrRepository.listByIds(qrSet).stream().collect(Collectors.toMap(QrDO::getId, o -> o, (o1, o2) -> o1));
        } else {
            qrMap = new HashMap<>();
        }
        List<AnswerDetailsDO> detailsDOList = detailsRepository.listBySheetIds(rows.stream().map(AnswerSheetDO::getId).collect(Collectors.toList()));
//        List<String> sids = detailsDOList.stream()
//                .filter(e -> e.getSubjectType() == 0 || e.getSubjectType() == 1)
//                .map(AnswerDetailsDO::getAnswerContent)
//                .flatMap(ids -> Arrays.stream(ids.split(",")))
//                .collect(Collectors.toList());
        Map<String, SubjectDO> sMap = subjectRepository.mapByIds(detailsDOList.stream().map(AnswerDetailsDO::getSubjectId).collect(Collectors.toList()));

        Map<Integer, List<String>> sidsMap = detailsDOList.stream()
                .filter(e -> e.getSubjectType() == 0 || e.getSubjectType() == 1)
                .collect(Collectors.groupingBy(AnswerDetailsDO::getSubjectType,
                        Collectors.mapping(AnswerDetailsDO::getAnswerContent, Collectors.toList())));

        List<String> sids = new ArrayList<>(sidsMap.getOrDefault(0, Collections.emptyList()));
        if (sidsMap.containsKey(1)) {
            sids.addAll(sidsMap.get(1)
                    .stream().flatMap(ids -> Arrays.stream(ids.split(",")))
                    .collect(Collectors.toList()));
        }
        Map<String, SubjectItemsDO> siMap = subjectItemsRepository.mapByIds(sids);

        return PageUtils.to(page, rows.stream().map(row -> {
            AnswerSheetVO vo = new AnswerSheetVO();
            BeanUtils.copyProperties(row, vo);
            vo.setSubmitTime(row.getUpdatedTime());
            if (qnMap.containsKey(row.getQnId())) {
                vo.setQnName(qnMap.get(row.getQnId()).getName());
            }
            if (qrMap.containsKey(row.getQrId())) {
                vo.setQrTip(qrMap.get(row.getQrId()).getTip());
            }
            vo.setDetails(detailsDOList.stream().filter(d -> d.getAnswerSheetId().equals(row.getId())).map(d -> {
                AnswerDetailsVO dv = new AnswerDetailsVO();
                BeanUtils.copyProperties(d, dv);
                dv.setType(d.getSubjectType());
                if (d.getSubjectType() == 2) {
                    dv.setAnswerContent(d.getAnswerContent());
                } else if (d.getSubjectType() == 1) {
                    dv.setAnswerContent(Arrays.stream(d.getAnswerContent().split(","))
                            .map(id -> (siMap.containsKey(id) ? siMap.get(id).getContent() : id))
                            .collect(Collectors.joining(",")));
                } else if (d.getSubjectType() == 0) {
                    dv.setAnswerContent(siMap.containsKey(d.getAnswerContent()) ? siMap.get(d.getAnswerContent()).getContent() : d.getAnswerContent());
                } else {
                    dv.setAnswerContent(d.getAnswerContent());
                }
                dv.setTitle(sMap.containsKey(d.getSubjectId()) ? sMap.get(d.getSubjectId()).getTitle() : "")
                        .setContent(sMap.containsKey(d.getSubjectId()) ? sMap.get(d.getSubjectId()).getContent() : "");
                return dv;
            }).collect(Collectors.toList()));
            return vo;
        }).collect(Collectors.toList()));
    }

    @Override
    @Transactional
    public Boolean saveSheet(AnswerSheetVO sheet) {
        AnswerSheetDO sheetDO = new AnswerSheetDO();
        BeanUtils.copyProperties(sheet, sheetDO);
        boolean saved = super.save(sheetDO);
        if (saved) {
            List<AnswerDetailsDO> details = sheet.getDetails().stream().map(d -> {
                AnswerDetailsDO detail = new AnswerDetailsDO();
                BeanUtils.copyProperties(d, detail);
                detail.setAnswerSheetId(sheetDO.getId());
                return detail;
            }).collect(Collectors.toList());
            if (!detailsRepository.saveBatch(details)) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return Boolean.FALSE;
            }
        } else {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    @Transactional
    public Boolean removeSheetByIds(Collection<String> sheetIds) {
        LambdaUpdateWrapper<AnswerDetailsDO> update = Wrappers.lambdaUpdate();
        update.in(AnswerDetailsDO::getAnswerSheetId, sheetIds);

        boolean removed = detailsRepository.remove(update);

        if (removed) {
            try {
                super.removeByIds(sheetIds);
                return Boolean.TRUE;
            } catch (Exception e) {
                // 如果删除失败，可以记录日志或进行其他处理
                throw new RuntimeException("删除答题详情失败 | ", e);
            }
        }

        return Boolean.FALSE;
    }
}
