package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.ExternalBackhaulDTO;
import cn.soul2.jyjc.admin.entity.ExternalBackhaulDO;
import cn.soul2.jyjc.admin.mapper.ExternalBackhaulMapper;
import cn.soul2.jyjc.admin.repository.IExternalBackhaulRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.ExternalBackhaulVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.stream.Collectors;

/**
 * <p>
 * 第三方结果回传表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:29:17
 */
@Service
public class ExternalBackhaulRepositoryImpl extends ServiceImpl<ExternalBackhaulMapper, ExternalBackhaulDO> implements IExternalBackhaulRepository {

    @Override
    public VPage<ExternalBackhaulVO> page(ExternalBackhaulDTO dto) {
        LambdaQueryWrapper<ExternalBackhaulDO> query = Wrappers.lambdaQuery();
        query.eq(ExternalBackhaulDO::getExternalName, dto.getExternalName())
                .likeRight(ExternalBackhaulDO::getOriginalBy, dto.getOriginalBy());
        VPage<ExternalBackhaulDO> page = PageUtils.to(super.page(PageUtils.build(dto), query));
        return PageUtils.to(page, page.getRows().stream().map(e -> {
            ExternalBackhaulVO vo = new ExternalBackhaulVO();
            BeanUtils.copyProperties(e, vo);
            return vo;
        }).collect(Collectors.toList()));
    }

    @Override
    public Boolean save(JSONObject jo, String externalName, String originalBy) {
        ExternalBackhaulDO backhaul = new ExternalBackhaulDO();
        backhaul.setExternalName(externalName)
                .setOriginalBackhaul(jo.toJSONString());
        if (StringUtils.isNotBlank(originalBy)) {
            backhaul.setOriginalBy(originalBy);
        }
        return super.save(backhaul);
    }
}
