package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.QrDTO;
import cn.soul2.jyjc.admin.entity.QrDO;
import cn.soul2.jyjc.admin.mapper.QrMapper;
import cn.soul2.jyjc.admin.repository.IQrRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.QrVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 二维码表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-11 14:28:45
 */
@Service
public class QrRepositoryImpl extends ServiceImpl<QrMapper, QrDO> implements IQrRepository {

    @Override
    public VPage<QrDO> page(QrDTO dto) {
        LambdaQueryWrapper<QrDO> query = Wrappers.lambdaQuery();
        query.likeRight(StringUtils.isNotBlank(dto.getTip()), QrDO::getTip, dto.getTip())
                .eq(StringUtils.isNotBlank(dto.getId()), QrDO::getId, dto.getId());
        if (dto.getStartTime() != null && dto.getEndTime() != null) {
            query.between(QrDO::getUpdatedTime, dto.getStartTime(), dto.getEndTime());
        } else {
            query.ge(dto.getStartTime() != null, QrDO::getUpdatedTime, dto.getStartTime())
                    .le(dto.getEndTime() != null, QrDO::getUpdatedTime, dto.getEndTime());
        }
        query.orderByDesc(QrDO::getUpdatedTime).orderByDesc(QrDO::getCreatedTime);
        return PageUtils.to(super.page(PageUtils.build(dto), query));
    }

    @Override
    public Boolean saveOrUpdate(Collection<QrVO> dtos) {
        List<QrDO> list = dtos.stream().map(dto -> {
            QrDO r = new QrDO();
            BeanUtils.copyProperties(dto, r);
            return r;
        }).collect(Collectors.toList());
        return super.saveOrUpdateBatch(list);
    }

    @Override
    public Boolean remove(Collection<String> ids) {
        return super.removeBatchByIds(ids);
    }

    @Override
    public boolean status(QrDTO status) {
        QrDO qr = new QrDO();
        BeanUtils.copyProperties(status, qr);
        return updateById(qr);
    }

    @Override
    public String getTo(String id) {
        QrDO qr = super.getById(id);
        if (qr != null) {
            if (StringUtils.isNotBlank(qr.getToUrl())) {
                return qr.getToUrl();
            }
            return qr.getQnId();
        } else {
            return null;
        }
    }
}
