package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.QnOptionsDTO;
import cn.soul2.jyjc.admin.dto.QuestionnaireDTO;
import cn.soul2.jyjc.admin.dto.base.UpdateStatusDTO;
import cn.soul2.jyjc.admin.entity.QuestionnaireDO;
import cn.soul2.jyjc.admin.mapper.QuestionnaireMapper;
import cn.soul2.jyjc.admin.repository.IQuestionnaireRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.QuestionnaireVO;
import cn.soul2.jyjc.admin.vo.base.OptionsVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 问卷表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-11 23:11:45
 */
@Service
public class QuestionnaireRepositoryImpl extends ServiceImpl<QuestionnaireMapper, QuestionnaireDO> implements IQuestionnaireRepository {

    @Resource
    private QuestionnaireMapper questionnaireMapper;

    @Override
    public VPage<QuestionnaireVO> page(QuestionnaireDTO dto) {
        LambdaQueryWrapper<QuestionnaireDO> query = Wrappers.lambdaQuery();
        query.likeRight(StringUtils.isNotBlank(dto.getName()), QuestionnaireDO::getName, dto.getName())
                .eq(StringUtils.isNotBlank(dto.getId()), QuestionnaireDO::getId, dto.getId());
        if (dto.getStartTime() != null && dto.getEndTime() != null) {
            query.between(QuestionnaireDO::getUpdatedTime, dto.getStartTime(), dto.getEndTime());
        } else {
            query.ge(dto.getStartTime() != null, QuestionnaireDO::getUpdatedTime, dto.getStartTime())
                    .le(dto.getEndTime() != null, QuestionnaireDO::getUpdatedTime, dto.getEndTime());
        }
        query.orderByDesc(QuestionnaireDO::getUpdatedTime).orderByDesc(QuestionnaireDO::getCreatedTime);
        VPage<QuestionnaireVO> page = PageUtils.to(questionnaireMapper.page(PageUtils.build(dto), query));
        return PageUtils.to(page, page.getRows().stream().map(e -> {
            QuestionnaireVO vo = new QuestionnaireVO();
            BeanUtils.copyProperties(e, vo);
            return vo;
        }).collect(Collectors.toList()));
    }

    @Override
    public Boolean saveOrUpdate(QuestionnaireDTO dto) {
        QuestionnaireDO entity = new QuestionnaireDO();
        BeanUtils.copyProperties(dto, entity);
        return super.saveOrUpdate(entity);
    }

    @Override
    public Boolean updateStatus(UpdateStatusDTO dto) {
        LambdaUpdateWrapper<QuestionnaireDO> update = Wrappers.lambdaUpdate();
        update.set(QuestionnaireDO::getStatus, dto.getStatus()).eq(QuestionnaireDO::getId, dto.getId());
        return super.update(update);
    }

    @Override
    public QuestionnaireDO getTheLatestOne() {
        LambdaQueryWrapper<QuestionnaireDO> query = Wrappers.lambdaQuery();
        query.eq(QuestionnaireDO::getStatus, 1);
        query.orderByDesc(QuestionnaireDO::getUpdatedTime);
        return super.getOne(query);
    }

    @Override
    public List<OptionsVO> options(QnOptionsDTO dto) {
        LambdaQueryWrapper<QuestionnaireDO> query = Wrappers.lambdaQuery();
        query.select(QuestionnaireDO::getId, QuestionnaireDO::getName)
                .eq(dto.getStatus() != null && dto.getStatus() > 0, QuestionnaireDO::getStatus, dto.getStatus())
                .orderByDesc(QuestionnaireDO::getUpdatedTime)
                .orderByDesc(QuestionnaireDO::getCreatedTime);
        return super.list(query).stream().map(q -> new OptionsVO().setCode(q.getId()).setLabel(q.getName()).setTime(q.getUpdatedTime())).collect(Collectors.toList());
    }

    @Override
    public Map<String, QuestionnaireDO> mapByIds(Collection<String> ids) {
        List<QuestionnaireDO> list = listByIds(ids);
        return list.stream().collect(Collectors.toMap(QuestionnaireDO::getId, o -> o, (o1, o2) -> o1));
    }
}
