package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.QnSubjectRefSaveDTO;
import cn.soul2.jyjc.admin.entity.RefQuestionnaireSubjectDO;
import cn.soul2.jyjc.admin.entity.SubjectDO;
import cn.soul2.jyjc.admin.mapper.RefQuestionnaireSubjectMapper;
import cn.soul2.jyjc.admin.repository.IRefQuestionnaireSubjectRepository;
import cn.soul2.jyjc.admin.repository.ISubjectRepository;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 题目-问卷连接表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
@Service
public class RefQuestionnaireSubjectRepositoryImpl extends ServiceImpl<RefQuestionnaireSubjectMapper, RefQuestionnaireSubjectDO> implements IRefQuestionnaireSubjectRepository {

    @Autowired
    private ISubjectRepository subjectRepository;

    @Override
    public List<SubjectDO> getSubjects(Collection<String> qnIds) {
        if (CollectionUtils.isNotEmpty(qnIds)) {
            LambdaQueryWrapper<RefQuestionnaireSubjectDO> query = Wrappers.lambdaQuery();
            query.in(RefQuestionnaireSubjectDO::getQnId, qnIds);
            Set<String> set = super.list(query).stream().map(RefQuestionnaireSubjectDO::getSubjectId).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(set)) {
                return subjectRepository.listByIds(set);
            } else {
                return new ArrayList<>();
            }
        } else {
            return new ArrayList<>();
        }
    }

    @Override
    @Transactional
    public Boolean updateRefSubjects(QnSubjectRefSaveDTO dto) {
        if (StringUtils.isBlank(dto.getQnId())) {
            return Boolean.FALSE;
        }
        // load old Ref data
        LambdaQueryWrapper<RefQuestionnaireSubjectDO> query = Wrappers.lambdaQuery();
        query.eq(RefQuestionnaireSubjectDO::getQnId, dto.getQnId());
        List<RefQuestionnaireSubjectDO> oldRefs = super.list(query);
        List<String> oldRefSubjectIds = oldRefs.stream().map(RefQuestionnaireSubjectDO::getSubjectId).collect(Collectors.toList());

        // Calculate the data that needs to be changed
        List<RefQuestionnaireSubjectDO> removeRefs = oldRefs.stream().filter(e -> !dto.getSubjectIds().contains(e.getSubjectId())).collect(Collectors.toList());
        List<String> insertRefs = dto.getSubjectIds().stream().filter(e -> !oldRefSubjectIds.contains(e)).collect(Collectors.toList());
        boolean removed;
        if (CollectionUtils.isNotEmpty(removeRefs)) {
            removed = super.removeByIds(removeRefs.stream().map(RefQuestionnaireSubjectDO::getId).collect(Collectors.toList()));
        } else {
            removed = true;
        }
        if (removed && CollectionUtils.isEmpty(insertRefs)) {
            return Boolean.TRUE;
        }
        if (removed) {
            boolean saved = super.saveBatch(insertRefs.stream().map(e -> {
                RefQuestionnaireSubjectDO ref = new RefQuestionnaireSubjectDO();
                return ref.setQnId(dto.getQnId()).setSubjectId(e);
            }).collect(Collectors.toList()));
            if (saved) {
                return Boolean.TRUE;
            }
        }
        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        return Boolean.FALSE;
    }
}
