package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.SubjectItemRefDTO;
import cn.soul2.jyjc.admin.entity.RefSubjectItemsDO;
import cn.soul2.jyjc.admin.entity.SubjectItemsDO;
import cn.soul2.jyjc.admin.mapper.RefSubjectItemsMapper;
import cn.soul2.jyjc.admin.repository.IRefSubjectItemsRepository;
import cn.soul2.jyjc.admin.repository.ISubjectItemsRepository;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 题目-选项连接表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
@Service
public class RefSubjectItemsRepositoryImpl extends ServiceImpl<RefSubjectItemsMapper, RefSubjectItemsDO> implements IRefSubjectItemsRepository {

    @Autowired
    private ISubjectItemsRepository itemsRepository;

    @Override
    public List<SubjectItemsDO> getSubjectItems(Collection<String> subjectIds) {
        if (CollectionUtils.isNotEmpty(subjectIds)) {
            LambdaQueryWrapper<RefSubjectItemsDO> query = Wrappers.lambdaQuery();
            query.in(RefSubjectItemsDO::getSubjectId, subjectIds);
            Set<String> set = super.list(query).stream().map(RefSubjectItemsDO::getItemId).collect(Collectors.toSet());
            if (CollectionUtils.isNotEmpty(set)) {
                return itemsRepository.listByIds(set);
            } else {
                return new ArrayList<>();
            }
        }
        return new ArrayList<>();
    }

    @Override
    public Boolean connectItems(SubjectItemRefDTO dto) {
        return super.saveBatch(dto.getItemIds().stream().map(s -> {
            RefSubjectItemsDO ref = new RefSubjectItemsDO();
            ref.setSubjectId(s).setSubjectId(dto.getSubjectId());
            return ref;
        }).collect(Collectors.toList()));
    }

    @Override
    @Transactional
    public Boolean disConnectItems(Collection<String> itemIds, Boolean andDestroy) {
        if (CollectionUtils.isNotEmpty(itemIds)) {
            LambdaUpdateWrapper<RefSubjectItemsDO> update = Wrappers.lambdaUpdate();
            update.in(RefSubjectItemsDO::getItemId, itemIds);
            if (andDestroy) {
                boolean itemRemoved = itemsRepository.removeByIds(itemIds);
                if (!itemRemoved) {
                    return Boolean.FALSE;
                }
            }
            boolean removed = super.remove(update);
            if (!removed) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            }
            return removed;
        }
        return Boolean.TRUE;
    }

    @Override
    public Boolean disConnectItems(String subjectId, Boolean andDestroy) {
        LambdaUpdateWrapper<RefSubjectItemsDO> update = Wrappers.lambdaUpdate();
        update.eq(RefSubjectItemsDO::getSubjectId, subjectId);
        if (andDestroy) {
            LambdaQueryWrapper<RefSubjectItemsDO> query = Wrappers.lambdaQuery();
            query.eq(RefSubjectItemsDO::getSubjectId, subjectId);
            boolean itemsRemoved = itemsRepository.removeByIds(super.list(query).stream().map(RefSubjectItemsDO::getItemId).collect(Collectors.toList()));
            if (!itemsRemoved) {
                return Boolean.FALSE;
            }
        }
        boolean removed = super.remove(update);
        if (!removed) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return removed;
    }


    @Override
    public Integer updateRef(String subjectId, Collection<String> newItemIds) {
        return null;
    }
}
