package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.entity.SubjectItemsDO;
import cn.soul2.jyjc.admin.mapper.SubjectItemsMapper;
import cn.soul2.jyjc.admin.repository.ISubjectItemsRepository;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 题选项表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
@Service
public class SubjectItemsRepositoryImpl extends ServiceImpl<SubjectItemsMapper, SubjectItemsDO> implements ISubjectItemsRepository {

    @Override
    public List<SubjectItemsDO> listBySubjectId(String subjectId) {
        if (StringUtils.isBlank(subjectId)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SubjectItemsDO> query = Wrappers.lambdaQuery();
        query.eq(SubjectItemsDO::getSubjectId, subjectId);
        return super.list(query);
    }

    @Override
    public List<SubjectItemsDO> listBySubjectIds(Collection<String> subjectIds) {
        if (CollectionUtils.isEmpty(subjectIds)) {
            return new ArrayList<>();
        }
        LambdaQueryWrapper<SubjectItemsDO> query = Wrappers.lambdaQuery();
        query.in(SubjectItemsDO::getSubjectId, subjectIds);
        return super.list(query);
    }

    @Override
    public Map<String, SubjectItemsDO> mapByIds(Collection<String> ids) {
        return super.listByIds(ids).stream().collect(Collectors.toMap(SubjectItemsDO::getId, o -> o, (o1, o2) -> o1));
    }

    @Override
    public Boolean removeBySubjectId(Collection<String> subjectIds) {
        LambdaUpdateWrapper<SubjectItemsDO> update = Wrappers.lambdaUpdate();
        update.in(SubjectItemsDO::getSubjectId, subjectIds);
        return super.remove(update);
    }
}
