package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.QnSubjectRefItemDTO;
import cn.soul2.jyjc.admin.dto.SubjectDTO;
import cn.soul2.jyjc.admin.dto.SubjectItemDTO;
import cn.soul2.jyjc.admin.dto.base.UpdateStatusDTO;
import cn.soul2.jyjc.admin.entity.SubjectDO;
import cn.soul2.jyjc.admin.entity.SubjectItemsDO;
import cn.soul2.jyjc.admin.entity.sqlresult.SubjectChooseListDO;
import cn.soul2.jyjc.admin.mapper.SubjectMapper;
import cn.soul2.jyjc.admin.repository.ISubjectItemsRepository;
import cn.soul2.jyjc.admin.repository.ISubjectRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import cn.soul2.jyjc.admin.vo.SubjectVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 题目表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-12 10:10:24
 */
@Service
public class SubjectRepositoryImpl extends ServiceImpl<SubjectMapper, SubjectDO> implements ISubjectRepository {

    @Autowired
    private ISubjectItemsRepository itemsRepository;

    @Resource
    private SubjectMapper subjectMapper;


    @Override
    @Transactional
    public Boolean saveOrUpdate(SubjectDTO dto) {
        SubjectDO entity = new SubjectDO();
        BeanUtils.copyProperties(dto, entity);
        if (!super.saveOrUpdate(entity)) {
            return Boolean.FALSE;
        }
        // If no id is passed, it is save.
        if (StringUtils.isBlank(entity.getId())) {
            if (CollectionUtils.isNotEmpty(dto.getItems())) {
                boolean itemsSaved = itemsRepository.saveBatch(dto.getItems().stream().map(e -> {
                    SubjectItemsDO item = new SubjectItemsDO();
                    BeanUtils.copyProperties(e, item);
                    item.setSubjectId(entity.getId());
                    return item;
                }).collect(Collectors.toList()));
                if (!itemsSaved) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return Boolean.FALSE;
                }
            }
            return Boolean.TRUE;
        } else {
            // otherwise update
            List<SubjectItemsDO> addOrUpdateItems = new ArrayList<>();
            List<String> removeItems = new ArrayList<>();

            List<SubjectItemsDO> oldItems = itemsRepository.listBySubjectId(entity.getId());
            Map<String, SubjectItemsDO> oldMap = oldItems.stream().collect(Collectors.toMap(SubjectItemsDO::getId, o -> o, (o1, o2) -> o1));
            Map<String, SubjectItemDTO> dtoMap = null;
            if (CollectionUtils.isNotEmpty(dto.getItems())) {
                dtoMap = dto.getItems().stream().collect(Collectors.toMap(SubjectItemDTO::getItemId, o -> o, (o1, o2) -> o1));
            } else {
                dtoMap = new HashMap<>(0);
            }
            dtoMap.forEach((k, v) -> {
                if (!oldMap.containsKey(k)) {
                    SubjectItemsDO addItem = new SubjectItemsDO();
                    BeanUtils.copyProperties(v, addItem);
                    addItem.setSubjectId(entity.getId());
                    addItem.setId(v.getItemId());
                    addOrUpdateItems.add(addItem);
                }
            });
            Map<String, SubjectItemDTO> finalDtoMap = dtoMap;
            oldMap.forEach((k, v) -> {
                if (!finalDtoMap.containsKey(k)) {
                    removeItems.add(k);
                } else {
                    SubjectItemsDO addItem = new SubjectItemsDO();
                    BeanUtils.copyProperties(v, addItem);
                    BeanUtils.copyProperties(finalDtoMap.get(k), addItem);
                    addOrUpdateItems.add(addItem);
                }
            });
            if (CollectionUtils.isNotEmpty(removeItems) && !itemsRepository.removeByIds(removeItems)) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return Boolean.FALSE;
            } else if (CollectionUtils.isNotEmpty(addOrUpdateItems) && !itemsRepository.saveOrUpdateBatch(addOrUpdateItems)) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return Boolean.FALSE;
            }
            return Boolean.TRUE;
        }
    }

    @Override
    @Transactional
    public Boolean removeAndItems(Collection<String> ids) {
        if (!super.removeByIds(ids) || !itemsRepository.removeBySubjectId(ids)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public VPage<SubjectVO> page(SubjectDTO dto) {
        LambdaQueryWrapper<SubjectDO> query = Wrappers.lambdaQuery();
        query.eq(dto.getType() != null, SubjectDO::getType, dto.getType())
                .likeRight(StringUtils.isNotBlank(dto.getTitle()), SubjectDO::getTitle, dto.getTitle())
                .orderByDesc(SubjectDO::getUpdatedTime);
        VPage<SubjectDO> dovPage = PageUtils.to(super.page(PageUtils.build(dto), query));
        return PageUtils.to(dovPage, dovPage.getRows().stream().map(e -> {
            SubjectVO vo = new SubjectVO();
            BeanUtils.copyProperties(e, vo);
            return vo;
        }).collect(Collectors.toList()));
    }

    @Override
    public Boolean status(UpdateStatusDTO dto) {
        LambdaUpdateWrapper<SubjectDO> update = Wrappers.lambdaUpdate();
        update.set(SubjectDO::getStatus, dto.getStatus()).eq(SubjectDO::getId, dto.getId());
        return super.update(update);
    }

    @Override
    public VPage<SubjectChooseListDO> chooseList(QnSubjectRefItemDTO dto) {
        LambdaQueryWrapper<SubjectDO> query = Wrappers.lambdaQuery();
        query.like(StringUtils.isNotBlank(dto.getTitle()), SubjectDO::getTitle, dto.getTitle())
                .eq(dto.getType() != null, SubjectDO::getType, dto.getType())
                .orderByDesc(SubjectDO::getUpdatedTime);
        return PageUtils.to(subjectMapper.useCounts(PageUtils.build(dto), query, dto.getQnId()));
    }

    @Override
    public Map<String, SubjectDO> mapByIds(Collection<String> ids) {
        return super.listByIds(ids).stream().collect(Collectors.toMap(SubjectDO::getId, o -> o, (o1, o2) -> o1));
    }
}
