package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.bean.UserLoginStatusBean;
import cn.soul2.jyjc.admin.dto.UserLoginDTO;
import cn.soul2.jyjc.admin.dto.UserLoginOutPageDTO;
import cn.soul2.jyjc.admin.dto.UserLogoutDTO;
import cn.soul2.jyjc.admin.entity.UserDO;
import cn.soul2.jyjc.admin.entity.UserLoginOutDO;
import cn.soul2.jyjc.admin.mapper.UserLoginOutMapper;
import cn.soul2.jyjc.admin.repository.IUserLoginOutRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户登入登出表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-29 20:46:42
 */
@Service
public class UserLoginOutRepositoryImpl extends ServiceImpl<UserLoginOutMapper, UserLoginOutDO> implements IUserLoginOutRepository {

    @Autowired
    private UserLoginStatusBean userLoginStatusBean;

    @Override
    public IPage<UserLoginOutDO> page(UserLoginOutPageDTO dto) {
        LambdaQueryWrapper<UserLoginOutDO> query = Wrappers.lambdaQuery();
        query.eq(StringUtils.isNotBlank(dto.getUserId()), UserLoginOutDO::getUserId, dto.getUserId())
                .orderByDesc(UserLoginOutDO::getCreatedTime);
        return super.page(PageUtils.build(dto), query);
    }

    @Override
    public String login(UserLoginDTO dto, UserDO user) {
        UserLoginOutDO loginOutDO;
        boolean saved;
        LocalDateTime now = LocalDateTime.now();
        // 判断是否登陆过,在缓存无法读取到目标用户名时即未登录,若登录过则令之前的登录记录失效
        String token = userLoginStatusBean.getTokenByUsername(dto.getUsername());
        if (token != null) {
            userLoginStatusBean.logout(token);
            UserLoginOutDO oldLoginOut = super.getById(token);
            oldLoginOut.setLapseTime(now).setLogoutTime(now);
            super.updateById(oldLoginOut);
        }
        loginOutDO = new UserLoginOutDO();
        loginOutDO.setUserId(user.getId())
                .setUserName(user.getUsername())
                .setLoginTime(now)
                .setLapseTime(now.plusHours(2))
        ;
        saved = super.save(loginOutDO);
        if (saved) {
            userLoginStatusBean.login(loginOutDO.getId(), loginOutDO);
            return loginOutDO.getId();
        }
        return null;
    }

    @Override
    public String logout(UserLogoutDTO dto) {
        // 清除缓存内的登录信息
        Boolean logout = userLoginStatusBean.logout(dto.getToken());
        // 保存退出登录的时间
        LocalDateTime now = LocalDateTime.now();
        UserLoginOutDO oldLoginOut = super.getById(dto.getToken());
        oldLoginOut.setLogoutTime(now);
        super.updateById(oldLoginOut);

        return logout ? dto.getUsername() : null;
    }

    @Override
    public Boolean logoutByUserId(String userId) {
        return userLoginStatusBean.logoutByUserId(userId);
    }

    @Override
    public Boolean removeByUserId(String userId) {
        if (org.apache.commons.lang3.StringUtils.isBlank(userId)) {
            return Boolean.TRUE;
        }
        LambdaUpdateWrapper<UserLoginOutDO> update = Wrappers.lambdaUpdate();
        update.eq(UserLoginOutDO::getUserId, userId);
        return super.remove(update);
    }

    @Override
    public List<UserLoginOutDO> getLoginedUser() {
        LocalDateTime now = LocalDateTime.now();
        LambdaQueryWrapper<UserLoginOutDO> query = Wrappers.lambdaQuery();
        query
                .isNull(UserLoginOutDO::getLogoutTime)
                .gt(UserLoginOutDO::getLapseTime, now);
        List<UserLoginOutDO> list = super.list(query);
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }

        return new ArrayList<>(list
                .stream()
                .collect(Collectors.toMap(UserLoginOutDO::getUserId,
                        Function.identity(),
                        BinaryOperator.maxBy(Comparator.comparing(UserLoginOutDO::getLoginTime))))
                .values());
    }
}
