package cn.soul2.jyjc.admin.repository.impl;

import cn.soul2.jyjc.admin.dto.UserPageDTO;
import cn.soul2.jyjc.admin.dto.UserSaveDTO;
import cn.soul2.jyjc.admin.dto.base.UpdateStatusDTO;
import cn.soul2.jyjc.admin.entity.UserDO;
import cn.soul2.jyjc.admin.mapper.UserMapper;
import cn.soul2.jyjc.admin.repository.IUserRepository;
import cn.soul2.jyjc.admin.utils.base.PageUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author Soul2
 * @since 2024-03-29 20:46:42
 */
@Service
public class UserRepositoryImpl extends ServiceImpl<UserMapper, UserDO> implements IUserRepository {

    @Override
    public IPage<UserDO> page(UserPageDTO dto) {
        LambdaQueryWrapper<UserDO> query = Wrappers.lambdaQuery();
        query.eq(dto.getStatus() != null, UserDO::getStatus, dto.getStatus())
                .like(StringUtils.isNotBlank(dto.getUsername()), UserDO::getUsername, dto.getUsername())
                .orderByDesc(UserDO::getUpdatedTime);
        return super.page(PageUtils.build(dto), query);
    }

    @Override
    public Boolean saveOrUpdate(UserSaveDTO dto) {
        UserDO user = new UserDO();
        BeanUtils.copyProperties(dto, user);
        return super.saveOrUpdate(user);
    }

    @Override
    public Boolean updateStatus(UpdateStatusDTO dto) {
        LambdaUpdateWrapper<UserDO> update = Wrappers.lambdaUpdate();
        update.set(UserDO::getStatus, dto.getStatus())
                .eq(UserDO::getId, dto.getId());
        return super.update(update);
    }

    @Override
    public UserDO getOneByUsername(String username) {
        LambdaQueryWrapper<UserDO> query = Wrappers.lambdaQuery();
        query.eq(UserDO::getUsername, username);
        return super.getOne(query, false);
    }
}
