package cn.soul2.jyjc.admin.service;

import cn.soul2.jyjc.admin.dto.UserLoginDTO;
import cn.soul2.jyjc.admin.dto.UserLogoutDTO;
import cn.soul2.jyjc.admin.entity.UserLoginOutDO;
import cn.soul2.jyjc.admin.vo.UserVO;

import java.util.List;

/**
 * @author Soul2
 * @date 2024-03-29 20:47
 */

public interface IUserService {

    /**
     * 注册一个用户
     *
     * @param dto 用户名和密码
     * @return {@link UserVO}
     */
    UserVO register(UserLoginDTO dto);

    /**
     * 用户登录
     *
     * @param dto 用户名和密码
     * @return {@link UserVO}
     */
    UserVO login(UserLoginDTO dto);

    /**
     * 用户退出登录
     *
     * @param dto 登录key
     * @return {@link Boolean}
     */
    Boolean logout(UserLogoutDTO dto);

    /**
     * 删除用户并退出登录,同时将登录记录一并删除
     *
     * @param userId 用户id
     * @return {@link Boolean}
     */
    Boolean remove(String userId);

    /**
     * 重建当前登录的用户
     *
     * @return {@link List}<{@link UserLoginOutDO}>
     */
    List<UserLoginOutDO> getLoginedUser();

}
