package cn.soul2.jyjc.admin.service.impl;

import cn.soul2.jyjc.admin.dto.AnswerPageQueryDTO;
import cn.soul2.jyjc.admin.dto.AnswerSubmitDTO;
import cn.soul2.jyjc.admin.entity.AnswerDetailsDO;
import cn.soul2.jyjc.admin.entity.AnswerSheetDO;
import cn.soul2.jyjc.admin.repository.IAnswerDetailsRepository;
import cn.soul2.jyjc.admin.repository.IAnswerSheetRepository;
import cn.soul2.jyjc.admin.service.IAnswerService;
import cn.soul2.jyjc.admin.vo.AnswerSheetVO;
import cn.soul2.jyjc.admin.vo.base.VPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Soul2
 * @date 2024-03-27 14:43
 */

@Service
public class AnswerServiceImpl implements IAnswerService {

    @Autowired
    private IAnswerSheetRepository sheetRepository;

    @Autowired
    private IAnswerDetailsRepository detailsRepository;

    @Override
    @Transactional
    public Boolean handleSubmit(AnswerSubmitDTO dto) {
        AnswerSheetDO sheet = new AnswerSheetDO();
        sheet.setRespondent(dto.getOpenid())
                .setQnId(dto.getQnId())
                .setQrId(dto.getQrId());
        boolean sheetSaved = sheetRepository.save(sheet);
        if (sheetSaved) {
            List<AnswerDetailsDO> details = dto.getDetails().stream().map(d -> {
                AnswerDetailsDO detail = new AnswerDetailsDO();
                detail.setAnswerContent(d.getContent())
                        .setAnswerSheetId(sheet.getId())
                        .setSubjectType(d.getSubjectType())
                        .setSubjectId(d.getSubjectId());
                return detail;
            }).collect(Collectors.toList());
            boolean detailsSaved = detailsRepository.saveBatch(details);
            if (!detailsSaved) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return Boolean.FALSE;
            }
        } else {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    @Override
    public VPage<AnswerSheetVO> page(AnswerPageQueryDTO dto) {
        return sheetRepository.page(dto);
    }
}
