package cn.soul2.jyjc.admin.service.impl;

import cn.soul2.jyjc.admin.dto.UserLoginDTO;
import cn.soul2.jyjc.admin.dto.UserLogoutDTO;
import cn.soul2.jyjc.admin.entity.UserDO;
import cn.soul2.jyjc.admin.entity.UserLoginOutDO;
import cn.soul2.jyjc.admin.repository.IUserLoginOutRepository;
import cn.soul2.jyjc.admin.repository.IUserRepository;
import cn.soul2.jyjc.admin.service.IUserService;
import cn.soul2.jyjc.admin.utils.Md5Utils;
import cn.soul2.jyjc.admin.vo.UserVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Soul2
 * @date 2024-03-29 20:47
 */

@Service
public class UserServiceImpl implements IUserService {

    @Autowired
    private IUserRepository userRepository;

    @Autowired
    private IUserLoginOutRepository loginOutRepository;

    @Override
    public UserVO register(UserLoginDTO dto) {
        UserVO vo = new UserVO();
        if (userRepository.getOneByUsername(dto.getUsername()) == null) {
            UserDO user = new UserDO();
            user.setUsername(dto.getUsername())
                    .setPassword(Md5Utils.md5(dto.getPassword()));
            boolean saved = userRepository.save(user);
            if (!saved) {
                vo.setLoginFail("注册失败!");
            } else {
                String loginKey = loginOutRepository.login(dto, user);
                if (loginKey != null) {
                    BeanUtils.copyProperties(user, vo);
                    vo.setToken(loginKey);
                } else {
                    vo.setLoginFail("注册成功但登录失败!");
                }
            }
        } else {
            vo.setLoginFail("用户名已存在!");
        }
        return vo;
    }

    @Override
    public UserVO login(UserLoginDTO dto) {
        UserVO vo = new UserVO();
        UserDO user = userRepository.getOneByUsername(dto.getUsername());
        if (user != null) {
            if (Md5Utils.verify(user.getPassword(), dto.getPassword())) {
                String loginKey = loginOutRepository.login(dto, user);
                if (loginKey != null) {
                    BeanUtils.copyProperties(user, vo);
                    vo.setToken(loginKey);
                } else {
                    vo.setLoginFail("登录失败!");
                }
            } else {
                vo.setLoginFail("密码错误!");
            }
        } else {
            vo.setLoginFail("该用户尚未注册!");
        }
        return vo;
    }

    @Override
    public Boolean logout(UserLogoutDTO dto) {
        return loginOutRepository.logout(dto) != null;
    }

    @Override
    public Boolean remove(String userId) {
        loginOutRepository.logoutByUserId(userId);
        loginOutRepository.removeByUserId(userId);
        return userRepository.removeById(userId);
    }

    @Override
    public List<UserLoginOutDO> getLoginedUser() {
        return loginOutRepository.getLoginedUser();
    }
}
